@@ -55,48 +55,32 @@
 #define GPG_FULL        4
 #define GPG_ULTIMATE    5
 
-typedef struct {
-	GCancellable *cancellable;
-	gpgme_ctx_t gctx;
-} key_op_generate_closure;
-
 static void
-key_op_generate_free (gpointer data)
+on_key_op_progress (void *opaque,
+                    const char *what,
+                    int type,
+                    int current,
+                    int total)
 {
-	key_op_generate_closure *closure = data;
-	g_clear_object (&closure->cancellable);
-	if (closure->gctx)
-		gpgme_release (closure->gctx);
-	g_free (closure);
-}
-
-static void
-on_key_op_generate_progress (void *opaque,
-                             const char *what,
-                             int type,
-                             int current,
-                             int total)
-{
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (opaque);
-	key_op_generate_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	seahorse_progress_update (closure->cancellable, res, "%s", what);
+    GTask *task = G_TASK (opaque);
+    seahorse_progress_update (g_task_get_cancellable (task), task, "%s", what);
 }
 
 static gboolean
 on_key_op_generate_complete (gpgme_error_t gerr,
                              gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	key_op_generate_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	GError *error = NULL;
+    GTask *task = G_TASK (user_data);
+    g_autoptr(GError) error = NULL;
 
-	if (seahorse_gpgme_propagate_error (gerr, &error)) {
-		g_simple_async_result_take_error (res, error);
-	}
+    if (seahorse_gpgme_propagate_error (gerr, &error)) {
+        g_task_return_error (task, g_steal_pointer (&error));
+        return FALSE; /* don't call again */
+    }
 
-	seahorse_progress_end (closure->cancellable, res);
-	g_simple_async_result_complete (res);
-	return FALSE; /* don't call again */
+    seahorse_progress_end (g_task_get_cancellable (task), task);
+    g_task_return_boolean (task, TRUE);
+    return FALSE; /* don't call again */
 }
 
 
@@ -126,98 +110,87 @@ seahorse_gpgme_key_op_generate_async (SeahorseGpgmeKeyring *keyring,
                                       GAsyncReadyCallback callback,
                                       gpointer user_data)
 {
-	gchar *common, *key_type, *start, *expires_date;
-	key_op_generate_closure *closure;
-	GSimpleAsyncResult *res;
-	GError *error = NULL;
-	const gchar *parms;
-	gpgme_error_t gerr = 0;
-	GSource *gsource;
+    const gchar* key_type;
+    g_autofree gchar *common = NULL, *start = NULL, *expires_date = NULL;
+    gpgme_ctx_t gctx;
+    g_autoptr(GTask) task = NULL;
+    g_autoptr(GError) error = NULL;
+    const gchar *parms;
+    gpgme_error_t gerr = 0;
+    g_autoptr(GSource) gsource = NULL;
 
-	g_return_if_fail (SEAHORSE_IS_GPGME_KEYRING (keyring));
-	g_return_if_fail (name);
-	g_return_if_fail (strlen (name) > 4);
-	g_return_if_fail (passphrase);
+    g_return_if_fail (SEAHORSE_IS_GPGME_KEYRING (keyring));
+    g_return_if_fail (name);
+    g_return_if_fail (strlen (name) > 4);
+    g_return_if_fail (passphrase);
 
-	/* Check lengths for each type */
-	switch (type) {
-	case DSA_ELGAMAL:
-		g_return_if_fail (length >= ELGAMAL_MIN || length <= LENGTH_MAX);
-		break;
-	case DSA:
-		g_return_if_fail (length >= DSA_MIN || length <= DSA_MAX);
-		break;
-	case RSA_RSA:
-	case RSA_SIGN:
-		g_return_if_fail (length >= RSA_MIN || length <= LENGTH_MAX);
-		break;
-	default:
-		g_return_if_reached ();
-		break;
-	}
+    /* Check lengths for each type */
+    switch (type) {
+    case DSA_ELGAMAL:
+        g_return_if_fail (length >= ELGAMAL_MIN || length <= LENGTH_MAX);
+        break;
+    case DSA:
+        g_return_if_fail (length >= DSA_MIN || length <= DSA_MAX);
+        break;
+    case RSA_RSA:
+    case RSA_SIGN:
+        g_return_if_fail (length >= RSA_MIN || length <= LENGTH_MAX);
+        break;
+    default:
+        g_return_if_reached ();
+        break;
+    }
 
-	if (expires != 0)
-		expires_date = seahorse_util_get_date_string (expires);
-	else
-		expires_date = g_strdup ("0");
+    if (expires != 0)
+        expires_date = seahorse_util_get_date_string (expires);
+    else
+        expires_date = g_strdup ("0");
 
-	/* Common xml */
-	common = g_strdup_printf ("Name-Real: %s\nExpire-Date: %s\nPassphrase: %s\n"
-			"</GnupgKeyParms>", name, expires_date, passphrase);
-	if (email != NULL && strlen (email) > 0)
-		common = g_strdup_printf ("Name-Email: %s\n%s", email, common);
-	if (comment != NULL && strlen (comment) > 0)
-		common = g_strdup_printf ("Name-Comment: %s\n%s", comment, common);
+    /* Common xml */
+    common = g_strdup_printf ("Name-Real: %s\nExpire-Date: %s\nPassphrase: %s\n"
+            "</GnupgKeyParms>", name, expires_date, passphrase);
+    if (email != NULL && strlen (email) > 0)
+        common = g_strdup_printf ("Name-Email: %s\n%s", email, common);
+    if (comment != NULL && strlen (comment) > 0)
+        common = g_strdup_printf ("Name-Comment: %s\n%s", comment, common);
 
-	if (type == DSA || type == DSA_ELGAMAL)
-		key_type = "Key-Type: DSA\nKey-Usage: sign";
-	else
-		key_type = "Key-Type: RSA\nKey-Usage: sign";
+    if (type == DSA || type == DSA_ELGAMAL)
+        key_type = "Key-Type: DSA\nKey-Usage: sign";
+    else
+        key_type = "Key-Type: RSA\nKey-Usage: sign";
 
-	start = g_strdup_printf ("<GnupgKeyParms format=\"internal\">\n%s\nKey-Length: ", key_type);
+    start = g_strdup_printf ("<GnupgKeyParms format=\"internal\">\n%s\nKey-Length: ", key_type);
 
-	/* Subkey xml */
-	if (type == DSA_ELGAMAL)
-		parms = g_strdup_printf ("%s%d\nSubkey-Type: ELG-E\nSubkey-Length: %d\nSubkey-Usage: encrypt\n%s",
-		                         start, (length < DSA_MAX) ? length : DSA_MAX, length, common);
-	else if (type == RSA_RSA)
-		parms = g_strdup_printf ("%s%d\nSubkey-Type: RSA\nSubkey-Length: %d\nSubkey-Usage: encrypt\n%s",
-		                         start, length, length, common);
-	else
-		parms = g_strdup_printf ("%s%d\n%s", start, length, common);
+    /* Subkey xml */
+    if (type == DSA_ELGAMAL)
+        parms = g_strdup_printf ("%s%d\nSubkey-Type: ELG-E\nSubkey-Length: %d\nSubkey-Usage: encrypt\n%s",
+                                 start, (length < DSA_MAX) ? length : DSA_MAX, length, common);
+    else if (type == RSA_RSA)
+        parms = g_strdup_printf ("%s%d\nSubkey-Type: RSA\nSubkey-Length: %d\nSubkey-Usage: encrypt\n%s",
+                                 start, length, length, common);
+    else
+        parms = g_strdup_printf ("%s%d\n%s", start, length, common);
 
-	res = g_simple_async_result_new (G_OBJECT (keyring), callback, user_data,
-	                                 seahorse_gpgme_key_op_generate_async);
+    gctx = seahorse_gpgme_keyring_new_context (&gerr);
 
-	closure = g_new0 (key_op_generate_closure, 1);
-	closure->gctx = seahorse_gpgme_keyring_new_context (&gerr);
-	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
-	gpgme_set_progress_cb (closure->gctx, on_key_op_generate_progress, res);
-	g_simple_async_result_set_op_res_gpointer (res, closure, key_op_generate_free);
+    task = g_task_new (keyring, cancellable, callback, user_data);
+    gpgme_set_progress_cb (gctx, on_key_op_progress, task);
+    g_task_set_task_data (task, gctx, (GDestroyNotify) gpgme_release);
 
-	seahorse_progress_prep_and_begin (cancellable, res, NULL);
-	gsource = seahorse_gpgme_gsource_new (closure->gctx, cancellable);
-	g_source_set_callback (gsource, (GSourceFunc)on_key_op_generate_complete,
-	                       g_object_ref (res), g_object_unref);
+    seahorse_progress_prep_and_begin (cancellable, task, NULL);
+    gsource = seahorse_gpgme_gsource_new (gctx, cancellable);
+    g_source_set_callback (gsource, (GSourceFunc)on_key_op_generate_complete,
+                           g_object_ref (task), g_object_unref);
 
-	if (gerr == 0)
-		gerr = gpgme_op_genkey_start (closure->gctx, parms, NULL, NULL);
+    if (gerr == 0)
+        gerr = gpgme_op_genkey_start (gctx, parms, NULL, NULL);
 
-	if (seahorse_gpgme_propagate_error (gerr, &error)) {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete_in_idle (res);
-	} else {
-		g_source_attach (gsource, g_main_context_default ());
-	}
+    if (seahorse_gpgme_propagate_error (gerr, &error)) {
+        g_task_return_error (task, g_steal_pointer (&error));
+        return;
+    }
 
-	g_source_unref (gsource);
-
-	/* Free xmls */
-	g_free (start);
-	g_free (common);
-	g_free (expires_date);
-
-	g_object_unref (res);
+    g_source_attach (gsource, g_main_context_default ());
 }
 
 gboolean
@@ -225,13 +198,9 @@ seahorse_gpgme_key_op_generate_finish (SeahorseGpgmeKeyring *keyring,
                                        GAsyncResult *result,
                                        GError **error)
 {
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (keyring),
-	                      seahorse_gpgme_key_op_generate_async), FALSE);
+    g_return_val_if_fail (g_task_is_valid (result, keyring), FALSE);
 
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
-		return FALSE;
-
-	return TRUE;
+    return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 /* helper function for deleting @skey */
@@ -631,8 +600,8 @@ seahorse_gpgme_key_op_sign_uid (SeahorseGpgmeUid *uid,  SeahorseGpgmeKey *signer
 	
 	seahorse_gpgme_key_get_private (signer);
 
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_UID (uid), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (signer), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_UID (uid), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (signer), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	
 	signing_key = seahorse_gpgme_key_get_private (signer);
 	g_return_val_if_fail (signing_key, GPG_E (GPG_ERR_WRONG_KEY_USAGE));
@@ -654,8 +623,8 @@ seahorse_gpgme_key_op_sign (SeahorseGpgmeKey *pkey, SeahorseGpgmeKey *signer,
 	
 	seahorse_gpgme_key_get_private (signer);
 
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (signer), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (signer), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	
 	signing_key = seahorse_gpgme_key_get_private (signer);
 	g_return_val_if_fail (signing_key, GPG_E (GPG_ERR_WRONG_KEY_USAGE));
@@ -665,125 +634,80 @@ seahorse_gpgme_key_op_sign (SeahorseGpgmeKey *pkey, SeahorseGpgmeKey *signer,
 	return sign_process (signed_key, signing_key, 0, check, options);
 }
 
-typedef enum {
-    PASS_START,
-    PASS_COMMAND,
-    PASS_PASSPHRASE,
-    PASS_QUIT,
-    PASS_SAVE,
-    PASS_ERROR
-} PassState;
-
-/* action helper for changing passphrase */
-static gpgme_error_t
-edit_pass_action (guint state, gpointer data, int fd)
+static gboolean
+on_key_op_change_pass_complete (gpgme_error_t gerr,
+                                gpointer user_data)
 {
-    switch (state) {
-    case PASS_COMMAND:
-        PRINT ((fd, "passwd"));
-        break;
-    case PASS_PASSPHRASE:
-        /* Do nothing */
-        return GPG_OK;
-    case PASS_QUIT:
-        PRINT ((fd, QUIT));
-        break;
-    case PASS_SAVE:
-        PRINT ((fd, YES));
-        break;
-    default:
-        return GPG_E (GPG_ERR_GENERAL);
+    GTask *task = G_TASK (user_data);
+    g_autoptr(GError) error = NULL;
+
+    if (seahorse_gpgme_propagate_error (gerr, &error)) {
+        g_task_return_error (task, g_steal_pointer (&error));
+        return FALSE; /* don't call again */
     }
-    
-    PRINT ((fd, "\n"));
-    return GPG_OK;
+
+    seahorse_progress_end (g_task_get_cancellable (task), task);
+    g_task_return_boolean (task, TRUE);
+    return FALSE; /* don't call again */
 }
 
-/* transition helper for changing passphrase */
-static guint
-edit_pass_transit (guint current_state, gpgme_status_code_t status,
-                   const gchar *args, gpointer data, gpgme_error_t *err)
+/**
+ * seahorse_gpgme_key_op_change_pass_async:
+ * @pkey: The key that you want to change the password of
+ * @cancellable: (nullable): A #GCancellable
+ * @callback: The callback that will be called when the operation finishes
+ * @user_data: (closure callback): User data passed on to @callback
+ *
+ * Changes the password of @pkey. The actual changing will be done by GPGME, so
+ * this function doesn't allow to specify the new password.
+ */
+void
+seahorse_gpgme_key_op_change_pass_async (SeahorseGpgmeKey *pkey,
+                                         GCancellable *cancellable,
+                                         GAsyncReadyCallback callback,
+                                         gpointer user_data)
 {
-    guint next_state;
-    
-    switch (current_state) {
-    /* start state, go to command */
-    case PASS_START:
-        if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
-            next_state = PASS_COMMAND;
-        else {
-            *err = GPG_E (GPG_ERR_GENERAL);
-            g_return_val_if_reached (PASS_ERROR);
-        }
-        break;
+    g_autoptr(GTask) task = NULL;
+    gpgme_ctx_t gctx;
+    gpgme_error_t gerr;
+    g_autoptr(GError) error = NULL;
+    g_autoptr(GSource) gsource = NULL;
+    gpgme_key_t key;
 
-    case PASS_COMMAND:
-        /* did command, go to should be the passphrase now */
-        if (status == GPGME_STATUS_NEED_PASSPHRASE_SYM)
-            next_state = PASS_PASSPHRASE;
-        
-        /* If all tries for passphrase were wrong, we get here */
-        else if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT)) {
-            *err = GPG_E (GPG_ERR_BAD_PASSPHRASE);
-            next_state = PASS_ERROR;
-        
-        /* No idea how we got here ... */
-        } else {
-            *err = GPG_E (GPG_ERR_GENERAL);
-            g_return_val_if_reached (PASS_ERROR);
-        }
-        break;
-    /* got passphrase now quit */
-    case PASS_PASSPHRASE:
-        if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
-            next_state = PASS_QUIT;
-        else {
-            *err = GPG_E (GPG_ERR_GENERAL);
-            g_return_val_if_reached (PASS_ERROR);
-        }
-        break;
-        
-    /* quit, go to save */
-    case PASS_QUIT:
-        if (status == GPGME_STATUS_GET_BOOL && g_str_equal (args, SAVE))
-            next_state = PASS_SAVE;
-        else {
-            *err = GPG_E (GPG_ERR_GENERAL);
-            g_return_val_if_reached (PASS_ERROR);
-        }
-        break;
-    
-    /* error, go to quit */
-    case PASS_ERROR:    
-        if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
-            next_state = PASS_QUIT;
-        else
-            next_state = PASS_ERROR;
-        break;
-    
-    default:
-        *err = GPG_E (GPG_ERR_GENERAL);
-        g_return_val_if_reached (PASS_ERROR);
-        break;
+    g_return_if_fail (SEAHORSE_GPGME_IS_KEY (pkey));
+    g_return_if_fail (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PRIVATE_KEY);
+
+    gctx = seahorse_gpgme_keyring_new_context (&gerr);
+
+    task = g_task_new (pkey, cancellable, callback, user_data);
+    gpgme_set_progress_cb (gctx, on_key_op_progress, task);
+    g_task_set_task_data (task, gctx, (GDestroyNotify) gpgme_release);
+
+    seahorse_progress_prep_and_begin (cancellable, task, NULL);
+    gsource = seahorse_gpgme_gsource_new (gctx, cancellable);
+    g_source_set_callback (gsource, (GSourceFunc)on_key_op_change_pass_complete,
+                           g_object_ref (task), g_object_unref);
+
+    key = seahorse_gpgme_key_get_private (pkey);
+    if (gerr == 0)
+        gerr = gpgme_op_passwd_start (gctx, key, 0);
+
+    if (seahorse_gpgme_propagate_error (gerr, &error)) {
+        g_task_return_error (task, g_steal_pointer (&error));
+        return;
     }
-    
-    return next_state;
+
+    g_source_attach (gsource, g_main_context_default ());
 }
 
-gpgme_error_t
-seahorse_gpgme_key_op_change_pass (SeahorseGpgmeKey *pkey)
+gboolean
+seahorse_gpgme_key_op_change_pass_finish (SeahorseGpgmeKey *pkey,
+                                          GAsyncResult *result,
+                                          GError **error)
 {
-	SeahorseEditParm *parms;
-	gpgme_error_t err;
-	
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
-	g_return_val_if_fail (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PRIVATE_KEY, GPG_E (GPG_ERR_WRONG_KEY_USAGE));
-	
-	parms = seahorse_edit_parm_new (PASS_START, edit_pass_action, edit_pass_transit, NULL);
-	
-	err = edit_key (pkey, parms);
-	
-	return err;
+    g_return_val_if_fail (g_task_is_valid (result, pkey), FALSE);
+
+    return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 typedef enum
@@ -917,7 +841,7 @@ seahorse_gpgme_key_op_set_trust (SeahorseGpgmeKey *pkey, SeahorseValidity trust)
 
 	g_debug ("[GPGME_KEY_OP] set_trust: trust = %i", trust);
 
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	g_return_val_if_fail (trust >= SEAHORSE_VALIDITY_NEVER, GPG_E (GPG_ERR_INV_VALUE));
 	g_return_val_if_fail (seahorse_gpgme_key_get_trust (pkey) != trust, GPG_E (GPG_ERR_INV_VALUE));
 	
@@ -1036,7 +960,7 @@ seahorse_gpgme_key_op_set_disabled (SeahorseGpgmeKey *pkey, gboolean disabled)
 	gchar *command;
 	SeahorseEditParm *parms;
 	
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	
 	/* Get command and op */
 	if (disabled)
@@ -1176,7 +1100,7 @@ seahorse_gpgme_key_op_set_expires (SeahorseGpgmeSubkey *subkey, const time_t exp
 	SeahorseEditParm *parms;
 	gpgme_key_t key;
 	
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	g_return_val_if_fail (expires != (time_t)seahorse_pgp_subkey_get_expires (SEAHORSE_PGP_SUBKEY (subkey)), GPG_E (GPG_ERR_INV_VALUE));
 	
 	key = seahorse_gpgme_subkey_get_pubkey (subkey);
@@ -1302,8 +1226,8 @@ seahorse_gpgme_key_op_add_revoker (SeahorseGpgmeKey *pkey, SeahorseGpgmeKey *rev
 	SeahorseEditParm *parms;
 	const gchar *keyid;
 
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (revoker), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (revoker), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
 	g_return_val_if_fail (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PRIVATE_KEY, GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	g_return_val_if_fail (seahorse_object_get_usage (SEAHORSE_OBJECT (revoker)) == SEAHORSE_USAGE_PRIVATE_KEY, GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 
@@ -1316,149 +1240,75 @@ seahorse_gpgme_key_op_add_revoker (SeahorseGpgmeKey *pkey, SeahorseGpgmeKey *rev
 	return edit_key (pkey, parms);
 }
 
-typedef enum {
-	ADD_UID_START,
-	ADD_UID_COMMAND,
-	ADD_UID_NAME,
-	ADD_UID_EMAIL,
-	ADD_UID_COMMENT,
-	ADD_UID_QUIT,
-	ADD_UID_SAVE,
-	ADD_UID_ERROR
-} AddUidState;
-
-typedef struct
+static gboolean
+on_key_op_add_uid_complete (gpgme_error_t gerr,
+                            gpointer user_data)
 {
-	const gchar	*name;
-	const gchar	*email;
-	const gchar	*comment;
-} UidParm;
+    GTask *task = G_TASK (user_data);
+    g_autoptr(GError) error = NULL;
 
-/* action helper for adding a new user ID */
-static gpgme_error_t
-add_uid_action (guint state, gpointer data, int fd)
-{
-	UidParm *parm = (UidParm*)data;
-	
-	switch (state) {
-		case ADD_UID_COMMAND:
-            PRINT ((fd, "adduid"));
-			break;
-		case ADD_UID_NAME:
-            PRINT ((fd, parm->name));
-			break;
-		case ADD_UID_EMAIL:
-            PRINT ((fd, parm->email));
-			break;
-		case ADD_UID_COMMENT:
-            PRINT ((fd, parm->comment));
-			break;
-		case ADD_UID_QUIT:
-            PRINT ((fd, QUIT));
-			break;
-		case ADD_UID_SAVE:
-            PRINT ((fd, YES));
-			break;
-		default:
-			return GPG_E (GPG_ERR_GENERAL);
-	}
-	
-    PRINT ((fd, "\n"));
-	return GPG_OK;
+    if (seahorse_gpgme_propagate_error (gerr, &error)) {
+        g_task_return_error (task, g_steal_pointer (&error));
+        return FALSE; /* don't call again */
+    }
+
+    seahorse_progress_end (g_task_get_cancellable (task), task);
+    g_task_return_boolean (task, TRUE);
+    return FALSE; /* don't call again */
 }
 
-/* transition helper for adding a new user ID */
-static guint
-add_uid_transit (guint current_state, gpgme_status_code_t status,
-		 const gchar *args, gpointer data, gpgme_error_t *err)
+void
+seahorse_gpgme_key_op_add_uid_async (SeahorseGpgmeKey *pkey,
+                                     const gchar *name,
+                                     const gchar *email,
+                                     const gchar *comment,
+                                     GCancellable *cancellable,
+                                     GAsyncReadyCallback callback,
+                                     gpointer user_data)
 {
-	guint next_state;
-	
-	switch (current_state) {
-		/* start, do command */
-		case ADD_UID_START:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
-				next_state = ADD_UID_COMMAND;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (ADD_UID_ERROR);
-			}
-			break;
-		/* did command, do name */
-		case ADD_UID_COMMAND:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, "keygen.name"))
-				next_state = ADD_UID_NAME;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (ADD_UID_ERROR);
-			}
-			break;
-		/* did name, do email */
-		case ADD_UID_NAME:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, "keygen.email"))
-				next_state = ADD_UID_EMAIL;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (ADD_UID_ERROR);
-			}
-			break;
-		/* did email, do comment */
-		case ADD_UID_EMAIL:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, "keygen.comment"))
-				next_state = ADD_UID_COMMENT;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (ADD_UID_ERROR);
-			}
-			break;
-		/* did comment, quit */
-		case ADD_UID_COMMENT:
-			next_state = ADD_UID_QUIT;
-			break;
-		/* quit, save */
-		case ADD_UID_QUIT:
-			if (status == GPGME_STATUS_GET_BOOL && g_str_equal (args, SAVE))
-				next_state = ADD_UID_SAVE;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (ADD_UID_ERROR);
-			}
-			break;
-		/* error, quit */
-		case ADD_UID_ERROR:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
-				next_state = ADD_UID_QUIT;
-			else
-				next_state = ADD_UID_ERROR;
-			break;
-		default:
-            *err = GPG_E (GPG_ERR_GENERAL);
-            g_return_val_if_reached (ADD_UID_ERROR);
-			break;
-	}
-	
-	return next_state;
+    g_autoptr(GTask) task = NULL;
+    gpgme_ctx_t gctx;
+    gpgme_error_t gerr;
+    g_autoptr(GError) error = NULL;
+    g_autoptr(GSource) gsource = NULL;
+    gpgme_key_t key;
+    g_autofree gchar* uid = NULL;
+
+    g_return_if_fail (SEAHORSE_GPGME_IS_KEY (pkey));
+    g_return_if_fail (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PRIVATE_KEY);
+
+    gctx = seahorse_gpgme_keyring_new_context (&gerr);
+
+    task = g_task_new (pkey, cancellable, callback, user_data);
+    gpgme_set_progress_cb (gctx, on_key_op_progress, task);
+    g_task_set_task_data (task, gctx, (GDestroyNotify) gpgme_release);
+
+    seahorse_progress_prep_and_begin (cancellable, task, NULL);
+    gsource = seahorse_gpgme_gsource_new (gctx, cancellable);
+    g_source_set_callback (gsource, (GSourceFunc)on_key_op_add_uid_complete,
+                           g_object_ref (task), g_object_unref);
+
+    key = seahorse_gpgme_key_get_private (pkey);
+    uid = seahorse_pgp_uid_calc_label (name, email, comment);
+    if (gerr == 0)
+        gerr = gpgme_op_adduid_start (gctx, key, uid, 0);
+
+    if (seahorse_gpgme_propagate_error (gerr, &error)) {
+        g_task_return_error (task, g_steal_pointer (&error));
+        return;
+    }
+
+    g_source_attach (gsource, g_main_context_default ());
 }
 
-gpgme_error_t
-seahorse_gpgme_key_op_add_uid (SeahorseGpgmeKey *pkey, const gchar *name, 
-                               const gchar *email, const gchar *comment)
+gboolean
+seahorse_gpgme_key_op_add_uid_finish (SeahorseGpgmeKey *pkey,
+                                      GAsyncResult *result,
+                                      GError **error)
 {
-	SeahorseEditParm *parms;
-	UidParm *uid_parm;
-	
-    g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
-    g_return_val_if_fail (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PRIVATE_KEY, GPG_E (GPG_ERR_WRONG_KEY_USAGE));
-	g_return_val_if_fail (name != NULL && strlen (name) >= 5, GPG_E (GPG_ERR_INV_VALUE));
-	
-	uid_parm = g_new (UidParm, 1);
-	uid_parm->name = name;
-	uid_parm->email = email;
-	uid_parm->comment = comment;
-	
-	parms = seahorse_edit_parm_new (ADD_UID_START, add_uid_action, add_uid_transit, uid_parm);
-	
-	return edit_key (pkey, parms);
+    g_return_val_if_fail (g_task_is_valid (result, pkey), FALSE);
+
+    return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 typedef enum {
@@ -1583,7 +1433,7 @@ seahorse_gpgme_key_op_add_subkey (SeahorseGpgmeKey *pkey, const SeahorseKeyEncTy
 	SeahorseKeyTypeTable table;
 	gpgme_error_t gerr;
 	
-    g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
+    g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
     g_return_val_if_fail (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PRIVATE_KEY, GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	
 	gerr = seahorse_gpgme_get_keytype_table (&table);
@@ -1731,7 +1581,7 @@ seahorse_gpgme_key_op_del_subkey (SeahorseGpgmeSubkey *subkey)
 	gpgme_key_t key;
 	int index;
 	
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	
 	key = seahorse_gpgme_subkey_get_pubkey (subkey);
 	g_return_val_if_fail (key, GPG_E (GPG_ERR_INV_VALUE));
@@ -1909,7 +1759,7 @@ seahorse_gpgme_key_op_revoke_subkey (SeahorseGpgmeSubkey *subkey, SeahorseRevoke
 	gpgme_subkey_t gsubkey;
 	gpgme_key_t key;
 	
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_SUBKEY (subkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	
 	gsubkey = seahorse_gpgme_subkey_get_subkey (subkey);
 	g_return_val_if_fail (!gsubkey->revoked, GPG_E (GPG_ERR_INV_VALUE));
@@ -2043,7 +1893,7 @@ seahorse_gpgme_key_op_primary_uid (SeahorseGpgmeUid *uid)
 	gpgme_user_id_t userid;
 	gpgme_key_t key;
 
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_UID (uid), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_UID (uid), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
     
 	/* Make sure not revoked */
 	userid = seahorse_gpgme_uid_get_userid (uid);
@@ -2192,7 +2042,7 @@ seahorse_gpgme_key_op_del_uid (SeahorseGpgmeUid *uid)
 	SeahorseEditParm *parms;
 	gpgme_key_t key;
 	
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_UID (uid), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_UID (uid), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
     
 	key = seahorse_gpgme_uid_get_pubkey (uid);
 	g_return_val_if_fail (key, GPG_E (GPG_ERR_INV_VALUE));
@@ -2320,7 +2170,7 @@ seahorse_gpgme_key_op_photo_add (SeahorseGpgmeKey *pkey, const gchar *filename)
 	PhotoIdAddParm photoid_add_parm;
 	gpgme_key_t key;
 	
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	g_return_val_if_fail (filename, GPG_E (GPG_ERR_INV_VALUE));
       
 	key = seahorse_gpgme_key_get_public (pkey);
@@ -2523,7 +2373,7 @@ seahorse_gpgme_key_op_photos_load (SeahorseGpgmeKey *pkey)
 	gchar *path;
 	gint fd;
 
-	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
+	g_return_val_if_fail (SEAHORSE_GPGME_IS_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	
 	key = seahorse_gpgme_key_get_public (pkey);
 	g_return_val_if_fail (key, GPG_E (GPG_ERR_INV_VALUE));
