@@ -127,7 +127,14 @@ gpgme_error_t         seahorse_gpgme_key_op_sign_uid         (SeahorseGpgmeUid *
                                                               SeahorseSignCheck check, 
                                                               SeahorseSignOptions options);
 
-gpgme_error_t         seahorse_gpgme_key_op_change_pass      (SeahorseGpgmeKey *pkey);
+void                 seahorse_gpgme_key_op_change_pass_async (SeahorseGpgmeKey *pkey,
+                                                              GCancellable *cancellable,
+                                                              GAsyncReadyCallback callback,
+                                                              gpointer user_data);
+
+gboolean            seahorse_gpgme_key_op_change_pass_finish (SeahorseGpgmeKey *pkey,
+                                                              GAsyncResult *Result,
+                                                              GError **error);
 
 gpgme_error_t         seahorse_gpgme_key_op_set_trust        (SeahorseGpgmeKey *pkey,
                                                               SeahorseValidity validity);
@@ -141,10 +148,17 @@ gpgme_error_t         seahorse_gpgme_key_op_set_expires      (SeahorseGpgmeSubke
 gpgme_error_t         seahorse_gpgme_key_op_add_revoker      (SeahorseGpgmeKey *pkey, 
                                                               SeahorseGpgmeKey *revoker);
 
-gpgme_error_t         seahorse_gpgme_key_op_add_uid          (SeahorseGpgmeKey *pkey,
+void                  seahorse_gpgme_key_op_add_uid_async    (SeahorseGpgmeKey *pkey,
                                                               const gchar *name,
                                                               const gchar *email,
-                                                              const gchar *comment);
+                                                              const gchar *comment,
+                                                              GCancellable *cancellable,
+                                                              GAsyncReadyCallback callback,
+                                                              gpointer user_data);
+
+gboolean              seahorse_gpgme_key_op_add_uid_finish  (SeahorseGpgmeKey *pkey,
+                                                             GAsyncResult *result,
+                                                             GError **error);
 
 gpgme_error_t         seahorse_gpgme_key_op_primary_uid      (SeahorseGpgmeUid *uid);
 
