@@ -39,7 +39,6 @@
 
 #include <gcr/gcr.h>
 
-#include <gio/gio.h>
 #include <glib/gi18n.h>
 
 #include <stdlib.h>
@@ -121,7 +120,9 @@ passphrase_get (gconstpointer dummy, const gchar *passphrase_hint,
 	return err;
 }
 
-struct _SeahorseGpgmeKeyringPrivate {
+struct _SeahorseGpgmeKeyring {
+    GObject parent_instance;
+
 	GHashTable *keys;
 	guint scheduled_refresh;                /* Source for refresh timeout */
 	GFileMonitor *monitor_handle;           /* For monitoring the .gnupg directory */
@@ -140,8 +141,6 @@ G_DEFINE_TYPE_WITH_CODE (SeahorseGpgmeKeyring, seahorse_gpgme_keyring, G_TYPE_OB
 
 typedef struct {
 	SeahorseGpgmeKeyring *keyring;
-	GCancellable *cancellable;
-	gulong cancelled_sig;
 	gpgme_ctx_t gctx;
 	GHashTable *checks;
 	gint parts;
@@ -156,9 +155,6 @@ keyring_list_free (gpointer data)
 		gpgme_release (closure->gctx);
 	if (closure->checks)
 		g_hash_table_destroy (closure->checks);
-	g_cancellable_disconnect (closure->cancellable,
-	                          closure->cancelled_sig);
-	g_clear_object (&closure->cancellable);
 	g_clear_object (&closure->keyring);
 	g_free (closure);
 }
@@ -196,7 +192,7 @@ add_key_to_context (SeahorseGpgmeKeyring *self,
 		pkey = seahorse_gpgme_key_new (SEAHORSE_PLACE (self), NULL, key);
 
 		/* Since we don't have a public key yet, save this away */
-		self->pv->orphan_secret = g_list_append (self->pv->orphan_secret, pkey);
+		self->orphan_secret = g_list_append (self->orphan_secret, pkey);
 
 		/* No key was loaded as far as everyone is concerned */
 		return NULL;
@@ -205,7 +201,7 @@ add_key_to_context (SeahorseGpgmeKeyring *self,
 	/* Just a new public key */
 
 	/* Check for orphans */
-	for (l = self->pv->orphan_secret; l; l = g_list_next (l)) {
+	for (l = self->orphan_secret; l; l = g_list_next (l)) {
 
 		seckey = seahorse_gpgme_key_get_private (l->data);
 		g_return_val_if_fail (seckey && seckey->subkeys && seckey->subkeys->keyid, NULL);
@@ -219,7 +215,7 @@ add_key_to_context (SeahorseGpgmeKeyring *self,
 			g_object_set (pkey, "pubkey", key, NULL);
 
 			/* Remove item from orphan list cleanly */
-			self->pv->orphan_secret = g_list_remove_link (self->pv->orphan_secret, l);
+			self->orphan_secret = g_list_remove_link (self->orphan_secret, l);
 			g_list_free (l);
 			break;
 		}
@@ -229,7 +225,7 @@ add_key_to_context (SeahorseGpgmeKeyring *self,
 		pkey = seahorse_gpgme_key_new (SEAHORSE_PLACE (self), key, NULL);
 
 	/* Add to context */
-	g_hash_table_insert (self->pv->keys, g_strdup (keyid), pkey);
+	g_hash_table_insert (self->keys, g_strdup (keyid), pkey);
 	gcr_collection_emit_added (GCR_COLLECTION (self), G_OBJECT (pkey));
 
 	return pkey;
@@ -243,7 +239,7 @@ remove_key (SeahorseGpgmeKeyring *self,
 {
 	SeahorseGpgmeKey *key;
 
-	key = g_hash_table_lookup (self->pv->keys, keyid);
+	key = g_hash_table_lookup (self->keys, keyid);
 	if (key != NULL)
 		seahorse_gpgme_keyring_remove_key (self, key);
 }
@@ -252,20 +248,19 @@ remove_key (SeahorseGpgmeKeyring *self,
 static gboolean
 on_idle_list_batch_of_keys (gpointer data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (data);
-	keyring_list_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	GTask *task = G_TASK (data);
+	keyring_list_closure *closure = g_task_get_task_data (task);
 	SeahorseGpgmeKey *pkey;
 	GHashTableIter iter;
 	gpgme_key_t key;
 	guint batch;
-	gchar *detail;
+	g_autofree gchar *detail = NULL;
 	const gchar *keyid;
 
 	/* We load until done if batch is zero */
 	batch = DEFAULT_LOAD_BATCH;
 
 	while (batch-- > 0) {
-
 		if (!GPG_IS_OK (gpgme_op_keylist_next (closure->gctx, &key))) {
 
 			gpgme_op_keylist_end (closure->gctx);
@@ -277,8 +272,8 @@ on_idle_list_batch_of_keys (gpointer data)
 					remove_key (closure->keyring, keyid);
 			}
 
-			seahorse_progress_end (closure->cancellable, res);
-			g_simple_async_result_complete (res);
+			seahorse_progress_end (g_task_get_cancellable (task), task);
+			g_task_return_boolean (task, TRUE);
 			return FALSE; /* Remove event handler */
 		}
 
@@ -286,10 +281,8 @@ on_idle_list_batch_of_keys (gpointer data)
 
 		/* During a refresh if only new or removed keys */
 		if (closure->checks) {
-
 			/* Make note that this key exists in key ring */
 			g_hash_table_remove (closure->checks, key->subkeys->keyid);
-
 		}
 
 		pkey = add_key_to_context (closure->keyring, key);
@@ -303,8 +296,7 @@ on_idle_list_batch_of_keys (gpointer data)
 	}
 
 	detail = g_strdup_printf (ngettext("Loaded %d key", "Loaded %d keys", closure->loaded), closure->loaded);
-	seahorse_progress_update (closure->cancellable, res, detail);
-	g_free (detail);
+	seahorse_progress_update (g_task_get_cancellable (task), task, detail);
 
 	return TRUE;
 }
@@ -313,8 +305,8 @@ static void
 on_keyring_list_cancelled (GCancellable *cancellable,
                            gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	keyring_list_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	GTask *task = G_TASK (user_data);
+	keyring_list_closure *closure = g_task_get_task_data (task);
 
 	gpgme_op_keylist_end (closure->gctx);
 }
@@ -328,23 +320,20 @@ seahorse_gpgme_keyring_list_async (SeahorseGpgmeKeyring *self,
                                    GAsyncReadyCallback callback,
                                    gpointer user_data)
 {
+	g_autoptr(GTask) task = NULL;
 	keyring_list_closure *closure;
-	GSimpleAsyncResult *res;
 	SeahorseObject *object;
 	gpgme_error_t gerr = 0;
 	GHashTableIter iter;
-	GError *error = NULL;
-	gchar *keyid;
+	g_autoptr(GError) error = NULL;
 
-	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
-	                                 seahorse_gpgme_keyring_list_async);
+	task = g_task_new (self, cancellable, callback, user_data);
 
 	closure = g_new0 (keyring_list_closure, 1);
 	closure->parts = parts;
 	closure->gctx = seahorse_gpgme_keyring_new_context (&gerr);
-	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
 	closure->keyring = g_object_ref (self);
-	g_simple_async_result_set_op_res_gpointer (res, closure, keyring_list_free);
+	g_task_set_task_data (task, closure, keyring_list_free);
 
 	/* Start the key listing */
 	if (closure->gctx) {
@@ -359,19 +348,18 @@ seahorse_gpgme_keyring_list_async (SeahorseGpgmeKeyring *self,
 
 	if (gerr != 0) {
 		seahorse_gpgme_propagate_error (gerr, &error);
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete_in_idle (res);
-		g_object_unref (res);
+		g_task_return_error (task, g_steal_pointer (&error));
 		return;
 	}
 
 	/* Loading all the keys? */
 	if (patterns == NULL) {
+		gchar *keyid;
 
 		closure->checks = g_hash_table_new_full (seahorse_pgp_keyid_hash,
 		                                         seahorse_pgp_keyid_equal,
 		                                         g_free, NULL);
-		g_hash_table_iter_init (&iter, self->pv->keys);
+		g_hash_table_iter_init (&iter, self->keys);
 		while (g_hash_table_iter_next (&iter, (gpointer *)&keyid, (gpointer *)&object)) {
 			if ((secret && seahorse_object_get_usage (object) == SEAHORSE_USAGE_PRIVATE_KEY) ||
 			    (!secret && seahorse_object_get_usage (object) == SEAHORSE_USAGE_PUBLIC_KEY)) {
@@ -381,16 +369,14 @@ seahorse_gpgme_keyring_list_async (SeahorseGpgmeKeyring *self,
 		}
 	}
 
-	seahorse_progress_prep_and_begin (cancellable, res, NULL);
+	seahorse_progress_prep_and_begin (cancellable, task, NULL);
 	if (cancellable)
-		closure->cancelled_sig = g_cancellable_connect (cancellable,
-		                                                G_CALLBACK (on_keyring_list_cancelled),
-		                                                res, NULL);
+		g_cancellable_connect (cancellable,
+		                       G_CALLBACK (on_keyring_list_cancelled),
+		                       task, NULL);
 
 	g_idle_add_full (G_PRIORITY_LOW, on_idle_list_batch_of_keys,
-	                 g_object_ref (res), g_object_unref);
-
-	g_object_unref (res);
+	                 g_steal_pointer (&task), g_object_unref);
 }
 
 static gboolean
@@ -398,22 +384,18 @@ seahorse_gpgme_keyring_list_finish (SeahorseGpgmeKeyring *keyring,
                                     GAsyncResult *result,
                                     GError **error)
 {
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (keyring),
-	                      seahorse_gpgme_keyring_list_async), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, keyring), FALSE);
 
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
-		return FALSE;
-
-	return TRUE;
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 static void
 cancel_scheduled_refresh (SeahorseGpgmeKeyring *self)
 {
-	if (self->pv->scheduled_refresh != 0) {
+	if (self->scheduled_refresh != 0) {
 		g_debug ("cancelling scheduled refresh event");
-		g_source_remove (self->pv->scheduled_refresh);
-		self->pv->scheduled_refresh = 0;
+		g_source_remove (self->scheduled_refresh);
+		self->scheduled_refresh = 0;
 	}
 }
 
@@ -422,7 +404,7 @@ scheduled_dummy (gpointer user_data)
 {
 	SeahorseGpgmeKeyring *self = SEAHORSE_GPGME_KEYRING (user_data);
 	g_debug ("dummy refresh event occurring now");
-	self->pv->scheduled_refresh = 0;
+	self->scheduled_refresh = 0;
 	return FALSE; /* don't run again */
 }
 
@@ -436,19 +418,20 @@ on_keyring_secret_list_complete (GObject *source,
                                  GAsyncResult *result,
                                  gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	keyring_load_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	GError *error = NULL;
-
-	if (!seahorse_gpgme_keyring_list_finish (SEAHORSE_GPGME_KEYRING (source),
-	                                         result, &error))
-		g_simple_async_result_take_error (res, error);
+	g_autoptr(GTask) task = G_TASK (user_data);
+	keyring_load_closure *closure = g_task_get_task_data (task);
+	g_autoptr(GError) error = NULL;
 
 	closure->secret_done = TRUE;
-	if (closure->public_done)
-		g_simple_async_result_complete (res);
 
-	g_object_unref (res);
+	if (!seahorse_gpgme_keyring_list_finish (SEAHORSE_GPGME_KEYRING (source),
+	                                         result, &error)) {
+		g_task_return_error (task, g_steal_pointer (&error));
+        return;
+    }
+
+	if (closure->public_done)
+		g_task_return_boolean (task, TRUE);
 }
 
 static void
@@ -456,19 +439,20 @@ on_keyring_public_list_complete (GObject *source,
                                  GAsyncResult *result,
                                  gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	keyring_load_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	GError *error = NULL;
-
-	if (!seahorse_gpgme_keyring_list_finish (SEAHORSE_GPGME_KEYRING (source),
-	                                         result, &error))
-		g_simple_async_result_take_error (res, error);
+	g_autoptr(GTask) task = G_TASK (user_data);
+	keyring_load_closure *closure = g_task_get_task_data (task);
+	g_autoptr(GError) error = NULL;
 
 	closure->public_done = TRUE;
-	if (closure->secret_done)
-		g_simple_async_result_complete (res);
 
-	g_object_unref (res);
+	if (!seahorse_gpgme_keyring_list_finish (SEAHORSE_GPGME_KEYRING (source),
+	                                         result, &error)) {
+		g_task_return_error (task, g_steal_pointer (&error));
+        return;
+    }
+
+	if (closure->secret_done)
+		g_task_return_boolean (task, TRUE);
 }
 
 static void
@@ -479,32 +463,29 @@ seahorse_gpgme_keyring_load_full_async (SeahorseGpgmeKeyring *self,
                                         GAsyncReadyCallback callback,
                                         gpointer user_data)
 {
-	GSimpleAsyncResult *res;
+	g_autoptr(GTask) task = NULL;
 	keyring_load_closure *closure;
 
 	/* Schedule a dummy refresh. This blocks all monitoring for a while */
 	cancel_scheduled_refresh (self);
-	self->pv->scheduled_refresh = g_timeout_add (500, scheduled_dummy, self);
+	self->scheduled_refresh = g_timeout_add (500, scheduled_dummy, self);
 	g_debug ("scheduled a dummy refresh");
 
 	g_debug ("refreshing keys...");
 
-	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
-	                                 seahorse_gpgme_keyring_load_full_async);
+	task = g_task_new (self, cancellable, callback, user_data);
 	closure = g_new0 (keyring_load_closure, 1);
-	g_simple_async_result_set_op_res_gpointer (res, closure, g_free);
+	g_task_set_task_data (task, closure, g_free);
 
 	/* Secret keys */
 	seahorse_gpgme_keyring_list_async (self, patterns, 0, TRUE, cancellable,
-	                                  on_keyring_secret_list_complete,
-	                                  g_object_ref (res));
+	                                   on_keyring_secret_list_complete,
+	                                   g_object_ref (task));
 
 	/* Public keys */
 	seahorse_gpgme_keyring_list_async (self, patterns, 0, FALSE, cancellable,
-	                                  on_keyring_public_list_complete,
-	                                  g_object_ref (res));
-
-	g_object_unref (res);
+	                                   on_keyring_public_list_complete,
+	                                   g_steal_pointer (&task));
 }
 
 SeahorseGpgmeKey *
@@ -514,7 +495,7 @@ seahorse_gpgme_keyring_lookup (SeahorseGpgmeKeyring *self,
 	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEYRING (self), NULL);
 	g_return_val_if_fail (keyid != NULL, NULL);
 
-	return g_hash_table_lookup (self->pv->keys, keyid);
+	return g_hash_table_lookup (self->keys, keyid);
 }
 
 void
@@ -524,13 +505,13 @@ seahorse_gpgme_keyring_remove_key (SeahorseGpgmeKeyring *self,
 	const gchar *keyid;
 
 	g_return_if_fail (SEAHORSE_IS_GPGME_KEYRING (self));
-	g_return_if_fail (SEAHORSE_IS_GPGME_KEY (key));
+	g_return_if_fail (SEAHORSE_GPGME_IS_KEY (key));
 
 	keyid = seahorse_pgp_key_get_keyid (SEAHORSE_PGP_KEY (key));
-	g_return_if_fail (g_hash_table_lookup (self->pv->keys, keyid) == key);
+	g_return_if_fail (g_hash_table_lookup (self->keys, keyid) == key);
 
 	g_object_ref (key);
-	g_hash_table_remove (self->pv->keys, keyid);
+	g_hash_table_remove (self->keys, keyid);
 	gcr_collection_emit_removed (GCR_COLLECTION (self), G_OBJECT (key));
 	g_object_unref (key);
 
@@ -552,35 +533,27 @@ seahorse_gpgme_keyring_load_finish (SeahorsePlace *place,
                                     GAsyncResult *result,
                                     GError **error)
 {
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (place),
-	                      seahorse_gpgme_keyring_load_full_async), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, place), FALSE);
 
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
-		return FALSE;
-
-	return TRUE;
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 typedef struct {
-	GCancellable *cancellable;
 	SeahorseGpgmeKeyring *keyring;
 	gpgme_ctx_t gctx;
 	gpgme_data_t data;
 	gchar **patterns;
-	GList *keys;
 } keyring_import_closure;
 
 static void
 keyring_import_free (gpointer data)
 {
 	keyring_import_closure *closure = data;
-	g_clear_object (&closure->cancellable);
 	if (closure->gctx)
 		gpgme_release (closure->gctx);
 	gpgme_data_release (closure->data);
 	g_object_unref (closure->keyring);
 	g_strfreev (closure->patterns);
-	g_list_free (closure->keys);
 	g_free (closure);
 }
 
@@ -589,33 +562,33 @@ on_keyring_import_loaded (GObject *source,
                          GAsyncResult *result,
                          gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	keyring_import_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	SeahorseObject *object;
+	g_autoptr(GTask) task = G_TASK (user_data);
+	keyring_import_closure *closure = g_task_get_task_data (task);
+    g_autoptr(GList) keys = NULL;
 	guint i;
 
 	for (i = 0; closure->patterns[i] != NULL; i++) {
-		object = g_hash_table_lookup (closure->keyring->pv->keys, closure->patterns[i]);
+	    SeahorseObject *object;
+
+		object = g_hash_table_lookup (closure->keyring->keys, closure->patterns[i]);
 		if (object == NULL) {
 			g_warning ("imported key but then couldn't find it in keyring: %s",
 			           closure->patterns[i]);
 			continue;
 		}
 
-		closure->keys = g_list_prepend (closure->keys, object);
+		keys = g_list_prepend (keys, object);
 	}
 
-	seahorse_progress_end (closure->cancellable, res);
-	g_simple_async_result_complete (res);
-	g_object_unref (res);
+	seahorse_progress_end (g_task_get_cancellable (task), result);
+	g_task_return_pointer (task, g_steal_pointer (&keys), (GDestroyNotify) g_list_free);
 }
 
 static gboolean
-on_keyring_import_complete (gpgme_error_t gerr,
-                           gpointer user_data)
+on_keyring_import_complete (gpgme_error_t gerr, gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	keyring_import_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	GTask *task = G_TASK (user_data);
+	keyring_import_closure *closure = g_task_get_task_data (task);
 	gpgme_import_result_t results;
 	gpgme_import_status_t import;
 	GError *error = NULL;
@@ -623,15 +596,14 @@ on_keyring_import_complete (gpgme_error_t gerr,
 	gint i;
 
 	if (seahorse_gpgme_propagate_error (gerr, &error)) {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete (res);
+		g_task_return_error (task, g_steal_pointer (&error));
 		return FALSE; /* don't call again */
 	}
 
 	/* Figure out which keys were imported */
 	results = gpgme_op_import_result (closure->gctx);
 	if (results == NULL) {
-		g_simple_async_result_complete (res);
+		g_task_return_pointer (task, NULL, NULL);
 		return FALSE; /* don't call again */
 	}
 
@@ -646,21 +618,20 @@ on_keyring_import_complete (gpgme_error_t gerr,
 
 	/* See if we've managed to import any ... */
 	if (closure->patterns[0] == NULL) {
-
 		/* ... try and find out why */
 		if (results->considered > 0 && results->no_user_id) {
 			msg = _("Invalid key data (missing UIDs). This may be due to a computer with a date set in the future or a missing self-signature.");
-			g_simple_async_result_set_error (res, SEAHORSE_ERROR, -1, "%s", msg);
+			g_task_return_new_error (task, SEAHORSE_ERROR, -1, "%s", msg);
 		}
 
-		g_simple_async_result_complete (res);
+		g_task_return_pointer (task, NULL, NULL);
 		return FALSE; /* don't call again */
 	}
 
 	/* Reload public keys */
 	seahorse_gpgme_keyring_load_full_async (closure->keyring, (const gchar **)closure->patterns,
-	                                        LOAD_FULL, closure->cancellable,
-	                                        on_keyring_import_loaded, g_object_ref (res));
+	                                        LOAD_FULL, g_task_get_cancellable (task),
+	                                        on_keyring_import_loaded, g_object_ref (task));
 
 	return FALSE; /* don't call again */
 }
@@ -672,39 +643,33 @@ seahorse_gpgme_keyring_import_async (SeahorseGpgmeKeyring *self,
                                      GAsyncReadyCallback callback,
                                      gpointer user_data)
 {
-	GSimpleAsyncResult *res;
+	g_autoptr(GTask) task = NULL;
 	keyring_import_closure *closure;
 	gpgme_error_t gerr = 0;
-	GError *error = NULL;
-	GSource *gsource = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GSource) gsource = NULL;
 
-	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
-	                                 seahorse_gpgme_keyring_import_async);
+	task = g_task_new (self, cancellable, callback, user_data);
 	closure = g_new0 (keyring_import_closure, 1);
-	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
 	closure->gctx = seahorse_gpgme_keyring_new_context (&gerr);
 	closure->data = seahorse_gpgme_data_input (input);
 	closure->keyring = g_object_ref (self);
-	g_simple_async_result_set_op_res_gpointer (res, closure, keyring_import_free);
+	g_task_set_task_data (task, closure, keyring_import_free);
 
 	if (gerr == 0) {
-		seahorse_progress_prep_and_begin (cancellable, res, NULL);
+		seahorse_progress_prep_and_begin (cancellable, task, NULL);
 		gsource = seahorse_gpgme_gsource_new (closure->gctx, cancellable);
 		g_source_set_callback (gsource, (GSourceFunc)on_keyring_import_complete,
-		                       g_object_ref (res), g_object_unref);
+		                       g_steal_pointer (&task), g_object_unref);
 		gerr = gpgme_op_import_start (closure->gctx, closure->data);
 	}
 
 	if (seahorse_gpgme_propagate_error (gerr, &error)) {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete_in_idle (res);
-
-	} else {
-		g_source_attach (gsource, g_main_context_default ());
+		g_task_return_error (task, g_steal_pointer (&error));
+        return;
 	}
 
-	g_source_unref (gsource);
-	g_object_unref (res);
+	g_source_attach (gsource, g_main_context_default ());
 }
 
 GList *
@@ -712,19 +677,9 @@ seahorse_gpgme_keyring_import_finish (SeahorseGpgmeKeyring *self,
                                       GAsyncResult *result,
                                       GError **error)
 {
-	keyring_import_closure *closure;
-	GList *results;
+	g_return_val_if_fail (g_task_is_valid (result, self), NULL);
 
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
-	                      seahorse_gpgme_keyring_import_async), NULL);
-
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
-		return NULL;
-
-	closure = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
-	results = closure->keys;
-	closure->keys = NULL;
-	return results;
+	return g_task_propagate_pointer (G_TASK (result), error);
 }
 
 static gboolean
@@ -752,9 +707,9 @@ monitor_gpg_homedir (GFileMonitor *handle, GFile *file, GFile *other_file,
 
 		name = g_file_get_basename (file);
 		if (g_str_has_suffix (name, ".gpg")) {
-			if (self->pv->scheduled_refresh == 0) {
+			if (self->scheduled_refresh == 0) {
 				g_debug ("scheduling refresh event due to file changes");
-				self->pv->scheduled_refresh = g_timeout_add (500, scheduled_refresh, self);
+				self->scheduled_refresh = g_timeout_add (500, scheduled_refresh, self);
 			}
 		}
 	}
@@ -767,30 +722,23 @@ seahorse_gpgme_keyring_init (SeahorseGpgmeKeyring *self)
 	const gchar *gpg_homedir;
 	GFile *file;
 
-	self->pv = G_TYPE_INSTANCE_GET_PRIVATE (self, SEAHORSE_TYPE_GPGME_KEYRING,
-	                                        SeahorseGpgmeKeyringPrivate);
-
-	self->pv->keys = g_hash_table_new_full (seahorse_pgp_keyid_hash,
+	self->keys = g_hash_table_new_full (seahorse_pgp_keyid_hash,
 	                                        seahorse_pgp_keyid_equal,
 	                                        g_free, g_object_unref);
 
-	/* init private vars */
-	self->pv = G_TYPE_INSTANCE_GET_PRIVATE (self, SEAHORSE_TYPE_GPGME_KEYRING,
-	                                        SeahorseGpgmeKeyringPrivate);
-
-	self->pv->scheduled_refresh = 0;
-	self->pv->monitor_handle = NULL;
+	self->scheduled_refresh = 0;
+	self->monitor_handle = NULL;
 
 	gpg_homedir = seahorse_gpg_homedir ();
 	if (gpg_homedir) {
 		file = g_file_new_for_path (gpg_homedir);
 		g_return_if_fail (file != NULL);
 
-		self->pv->monitor_handle = g_file_monitor_directory (file, G_FILE_MONITOR_NONE, NULL, &err);
+		self->monitor_handle = g_file_monitor_directory (file, G_FILE_MONITOR_NONE, NULL, &err);
 		g_object_unref (file);
 
-		if (self->pv->monitor_handle) {
-			g_signal_connect (self->pv->monitor_handle, "changed",
+		if (self->monitor_handle) {
+			g_signal_connect (self->monitor_handle, "changed",
 			                  G_CALLBACK (monitor_gpg_homedir), self);
 		} else {
 			g_warning ("couldn't monitor the GPG home directory: %s: %s",
@@ -826,8 +774,8 @@ static GtkActionGroup *
 seahorse_gpgme_keyring_get_actions (SeahorsePlace *place)
 {
 	SeahorseGpgmeKeyring *self = SEAHORSE_GPGME_KEYRING (place);
-	if (self->pv->actions)
-		return g_object_ref (self->pv->actions);
+	if (self->actions)
+		return g_object_ref (self->actions);
 	return NULL;
 }
 
@@ -891,20 +839,16 @@ seahorse_gpgme_keyring_dispose (GObject *object)
 	SeahorseGpgmeKeyring *self = SEAHORSE_GPGME_KEYRING (object);
 	GList *l;
 
-	if (self->pv->actions)
-		gtk_action_group_set_sensitive (self->pv->actions, TRUE);
-	g_hash_table_remove_all (self->pv->keys);
+	if (self->actions)
+		gtk_action_group_set_sensitive (self->actions, TRUE);
+	g_hash_table_remove_all (self->keys);
 
 	cancel_scheduled_refresh (self);
-	if (self->pv->monitor_handle) {
-		g_object_unref (self->pv->monitor_handle);
-		self->pv->monitor_handle = NULL;
-	}
+	g_clear_object (&self->monitor_handle);
 
-	for (l = self->pv->orphan_secret; l != NULL; l = g_list_next (l))
+	for (l = self->orphan_secret; l != NULL; l = g_list_next (l))
 		g_object_unref (l->data);
-	g_list_free (self->pv->orphan_secret);
-	self->pv->orphan_secret = NULL;
+	g_clear_pointer (&self->orphan_secret, g_list_free);
 
 	G_OBJECT_CLASS (seahorse_gpgme_keyring_parent_class)->dispose (object);
 }
@@ -914,12 +858,12 @@ seahorse_gpgme_keyring_finalize (GObject *object)
 {
 	SeahorseGpgmeKeyring *self = SEAHORSE_GPGME_KEYRING (object);
 
-	g_clear_object (&self->pv->actions);
-	g_hash_table_destroy (self->pv->keys);
+	g_clear_object (&self->actions);
+	g_hash_table_destroy (self->keys);
 
 	/* All monitoring and scheduling should be done */
-	g_assert (self->pv->scheduled_refresh == 0);
-	g_assert (self->pv->monitor_handle == 0);
+	g_assert (self->scheduled_refresh == 0);
+	g_assert (self->monitor_handle == 0);
 
 	G_OBJECT_CLASS (seahorse_gpgme_keyring_parent_class)->finalize (object);
 }
@@ -942,8 +886,6 @@ seahorse_gpgme_keyring_class_init (SeahorseGpgmeKeyringClass *klass)
 	gobject_class->dispose = seahorse_gpgme_keyring_dispose;
 	gobject_class->finalize = seahorse_gpgme_keyring_finalize;
 
-	g_type_class_add_private (klass, sizeof (SeahorseGpgmeKeyringPrivate));
-
 	g_object_class_override_property (gobject_class, PROP_LABEL, "label");
 	g_object_class_override_property (gobject_class, PROP_DESCRIPTION, "description");
 	g_object_class_override_property (gobject_class, PROP_URI, "uri");
@@ -968,14 +910,14 @@ static guint
 seahorse_gpgme_keyring_get_length (GcrCollection *collection)
 {
 	SeahorseGpgmeKeyring *self = SEAHORSE_GPGME_KEYRING (collection);
-	return g_hash_table_size (self->pv->keys);
+	return g_hash_table_size (self->keys);
 }
 
 static GList *
 seahorse_gpgme_keyring_get_objects (GcrCollection *collection)
 {
 	SeahorseGpgmeKeyring *self = SEAHORSE_GPGME_KEYRING (collection);
-	return g_hash_table_get_values (self->pv->keys);
+	return g_hash_table_get_values (self->keys);
 }
 
 static gboolean
@@ -985,11 +927,11 @@ seahorse_gpgme_keyring_contains (GcrCollection *collection,
 	SeahorseGpgmeKeyring *self = SEAHORSE_GPGME_KEYRING (collection);
 	const gchar *keyid;
 
-	if (!SEAHORSE_IS_GPGME_KEY (object))
+	if (!SEAHORSE_GPGME_IS_KEY (object))
 		return FALSE;
 
 	keyid = seahorse_pgp_key_get_keyid (SEAHORSE_PGP_KEY (object));
-	return g_hash_table_lookup (self->pv->keys, keyid) == object;
+	return g_hash_table_lookup (self->keys, keyid) == object;
 }
 
 static void
