@@ -32,27 +32,10 @@
 
 #ifdef WITH_HKP
 
-#define SEAHORSE_TYPE_HKP_SOURCE            (seahorse_hkp_source_get_type ())
-#define SEAHORSE_HKP_SOURCE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SEAHORSE_TYPE_HKP_SOURCE, SeahorseHKPSource))
-#define SEAHORSE_HKP_SOURCE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SEAHORSE_TYPE_HKP_SOURCE, SeahorseHKPSourceClass))
-#define SEAHORSE_IS_HKP_SOURCE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SEAHORSE_TYPE_HKP_SOURCE))
-#define SEAHORSE_IS_HKP_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SEAHORSE_TYPE_HKP_SOURCE))
-#define SEAHORSE_HKP_SOURCE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SEAHORSE_TYPE_HKP_SOURCE, SeahorseHKPSourceClass))
-
-typedef struct _SeahorseHKPSource SeahorseHKPSource;
-typedef struct _SeahorseHKPSourceClass SeahorseHKPSourceClass;
-
-struct _SeahorseHKPSource {
-    SeahorseServerSource parent;
-    
-    /*< private >*/
-};
-
-struct _SeahorseHKPSourceClass {
-    SeahorseServerSourceClass parent_class;
-};
-
-GType                 seahorse_hkp_source_get_type (void);
+#define SEAHORSE_TYPE_HKP_SOURCE (seahorse_hkp_source_get_type ())
+G_DECLARE_FINAL_TYPE (SeahorseHKPSource, seahorse_hkp_source,
+                      SEAHORSE, HKP_SOURCE,
+                      SeahorseServerSource)
 
 SeahorseHKPSource*    seahorse_hkp_source_new      (const gchar *uri, 
                                                     const gchar *host);
