@@ -48,17 +48,14 @@ enum {
 static SeahorsePgpBackend *pgp_backend = NULL;
 
 struct _SeahorsePgpBackend {
-	GObject parent;
-	SeahorseGpgmeKeyring *keyring;
-	SeahorseDiscovery *discovery;
-	SeahorseUnknownSource *unknown;
-	GHashTable *remotes;
-	GtkActionGroup *actions;
-	gboolean loaded;
-};
+    GObject parent;
 
-struct _SeahorsePgpBackendClass {
-	GObjectClass parent_class;
+    SeahorseGpgmeKeyring *keyring;
+    SeahorseDiscovery *discovery;
+    SeahorseUnknownSource *unknown;
+    GHashTable *remotes;
+    SeahorseActionGroup *actions;
+    gboolean loaded;
 };
 
 static void         seahorse_pgp_backend_iface            (SeahorseBackendIface *iface);
@@ -80,8 +77,6 @@ seahorse_pgp_backend_init (SeahorsePgpBackend *self)
 	                                       g_free, g_object_unref);
 
 	self->actions = seahorse_pgp_backend_actions_instance ();
-
-	seahorse_gpgme_generate_register ();
 }
 
 #ifdef WITH_KEYSERVER
@@ -198,7 +193,7 @@ seahorse_pgp_backend_get_description (SeahorseBackend *backend)
 	return _("PGP keys are for encrypting email or files");
 }
 
-static GtkActionGroup *
+static SeahorseActionGroup *
 seahorse_pgp_backend_get_actions (SeahorseBackend *backend)
 {
 	SeahorsePgpBackend *self = SEAHORSE_PGP_BACKEND (backend);
@@ -208,7 +203,7 @@ seahorse_pgp_backend_get_actions (SeahorseBackend *backend)
 static gboolean
 seahorse_pgp_backend_get_loaded (SeahorseBackend *backend)
 {
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (backend), FALSE);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (backend), FALSE);
 
 	return SEAHORSE_PGP_BACKEND (backend)->loaded;
 }
@@ -342,7 +337,7 @@ seahorse_pgp_backend_initialize (void)
 	SeahorsePgpBackend *self;
 
 	g_return_if_fail (pgp_backend == NULL);
-	self = g_object_new (SEAHORSE_TYPE_PGP_BACKEND, NULL);
+	self = g_object_new (SEAHORSE_PGP_TYPE_BACKEND, NULL);
 
 	seahorse_backend_register (SEAHORSE_BACKEND (self));
 	g_object_unref (self);
@@ -356,7 +351,7 @@ SeahorseGpgmeKeyring *
 seahorse_pgp_backend_get_default_keyring (SeahorsePgpBackend *self)
 {
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (self), NULL);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (self), NULL);
 	g_return_val_if_fail (self->keyring, NULL);
 	return self->keyring;
 }
@@ -370,7 +365,7 @@ seahorse_pgp_backend_get_default_key (SeahorsePgpBackend *self)
 	gchar *value;
 
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (self), NULL);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (self), NULL);
 
 	settings = seahorse_pgp_settings_instance ();
 	if (settings != NULL) {
@@ -394,7 +389,7 @@ SeahorseDiscovery *
 seahorse_pgp_backend_get_discovery (SeahorsePgpBackend *self)
 {
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (self), NULL);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (self), NULL);
 	g_return_val_if_fail (self->discovery, NULL);
 	return self->discovery;
 }
@@ -404,7 +399,7 @@ seahorse_pgp_backend_lookup_remote (SeahorsePgpBackend *self,
                                     const gchar *uri)
 {
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (self), NULL);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (self), NULL);
 
 	return g_hash_table_lookup (self->remotes, uri);
 }
@@ -415,7 +410,7 @@ seahorse_pgp_backend_add_remote (SeahorsePgpBackend *self,
                                  SeahorseServerSource *source)
 {
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_if_fail (SEAHORSE_IS_PGP_BACKEND (self));
+	g_return_if_fail (SEAHORSE_PGP_IS_BACKEND (self));
 	g_return_if_fail (uri != NULL);
 	g_return_if_fail (SEAHORSE_IS_SERVER_SOURCE (source));
 	g_return_if_fail (g_hash_table_lookup (self->remotes, uri) == NULL);
@@ -428,7 +423,7 @@ seahorse_pgp_backend_remove_remote (SeahorsePgpBackend *self,
                                     const gchar *uri)
 {
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_if_fail (SEAHORSE_IS_PGP_BACKEND (self));
+	g_return_if_fail (SEAHORSE_PGP_IS_BACKEND (self));
 	g_return_if_fail (uri != NULL);
 
 	g_hash_table_remove (self->remotes, uri);
@@ -491,7 +486,7 @@ seahorse_pgp_backend_search_remote_async (SeahorsePgpBackend *self,
 	guint i;
 
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_if_fail (SEAHORSE_IS_PGP_BACKEND (self));
+	g_return_if_fail (SEAHORSE_PGP_IS_BACKEND (self));
 
 	/* Get a list of all selected key servers */
 	names = g_settings_get_strv (G_SETTINGS (seahorse_app_settings_instance ()), "last-search-servers");
@@ -544,7 +539,7 @@ seahorse_pgp_backend_search_remote_finish (SeahorsePgpBackend *self,
 	GSimpleAsyncResult *res;
 
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (self), FALSE);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (self), FALSE);
 	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
 	                      seahorse_pgp_backend_search_remote_async), FALSE);
 
@@ -609,7 +604,7 @@ seahorse_pgp_backend_transfer_async (SeahorsePgpBackend *self,
 	GList *next;
 
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_if_fail (SEAHORSE_IS_PGP_BACKEND (self));
+	g_return_if_fail (SEAHORSE_PGP_IS_BACKEND (self));
 	g_return_if_fail (SEAHORSE_IS_PLACE (to));
 
 	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
@@ -663,7 +658,7 @@ seahorse_pgp_backend_transfer_finish (SeahorsePgpBackend *self,
 	GSimpleAsyncResult *res;
 
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (self), FALSE);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (self), FALSE);
 	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
 	                      seahorse_pgp_backend_transfer_async), FALSE);
 
@@ -689,7 +684,7 @@ seahorse_pgp_backend_retrieve_async (SeahorsePgpBackend *self,
 	GHashTableIter iter;
 
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_if_fail (SEAHORSE_IS_PGP_BACKEND (self));
+	g_return_if_fail (SEAHORSE_PGP_IS_BACKEND (self));
 	g_return_if_fail (SEAHORSE_IS_PLACE (to));
 
 	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
@@ -723,7 +718,7 @@ seahorse_pgp_backend_retrieve_finish (SeahorsePgpBackend *self,
 	GSimpleAsyncResult *res;
 
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (self), FALSE);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (self), FALSE);
 	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
 	                      seahorse_pgp_backend_retrieve_async), FALSE);
 
@@ -750,7 +745,7 @@ seahorse_pgp_backend_discover_keys (SeahorsePgpBackend *self,
 	gint i;
 
 	self = self ? self : seahorse_pgp_backend_get ();
-	g_return_val_if_fail (SEAHORSE_IS_PGP_BACKEND (self), NULL);
+	g_return_val_if_fail (SEAHORSE_PGP_IS_BACKEND (self), NULL);
 
 	todiscover = g_ptr_array_new ();
 
